using System.Diagnostics;
using System.Data;
using System.Collections;
using Microsoft.VisualBasic;
using System.Collections.Generic;
using System;

namespace Optim_PowellSearch1
{
	public delegate double MyFxDelegate(int nNumVars, ref double[] fX, ref double[] fParam);
	public delegate string SayFxDelegate();
	
	public class CPowellSearch1
	{
		
		MyFxDelegate m_MyFx;
		
		protected double MyFxEx(int nNumVars, ref double[] fX, ref double[] fParam, ref double[,] fSmat, int nII, double fAlpha)
		{
			int i;
			double[] fXX = new double[nNumVars + 1];
			
			for (i = 0; i < nNumVars; i++)
			{
				fXX[i] = fX[i] + fAlpha * fSmat[nII, i];
			}

            return m_MyFx(nNumVars, ref fXX, ref fParam);
		}
		
		protected bool LinSearch_DirectSearch(int nNumVars, ref double[] fX, ref double[] fParam, ref double fAlpha, ref double[,] fSmat, int nII, double fInitStep, double fMinStep)
		{
            double F1, F2;
			
			F1 = MyFxEx(nNumVars, ref fX, ref fParam, ref fSmat, nII, fAlpha);
			
			do
			{
				F2 = MyFxEx(nNumVars, ref fX, ref fParam, ref fSmat, nII, fAlpha + fInitStep);
				if (F2 < F1)
				{
					F1 = F2;
					fAlpha += fInitStep;
				}
				else
				{
					F2 = MyFxEx(nNumVars, ref fX, ref fX, ref fSmat, nII, fAlpha - fInitStep);
					if (F2 < F1)
					{
						F1 = F2;
						fAlpha -= fInitStep;
					}
					else
					{
						// reduce search step size
						fInitStep /= 10;
					}
				}
			} while (!(fInitStep < fMinStep));
			
			return true;
			
		}
		
		
		public double CalcOptim(int nNumVars, ref double[] fX, ref double[] fParam, double fEpsFx, double fEpsStep, int nMaxCycles, ref int nIter, ref string sErrorMsg, MyFxDelegate MyFx)
		{

            int i, j, k, m; 
			double fAlpha, fSum, F1, F2;
			double[] fX1 = new double[nNumVars];
			double[] fX2 = new double[nNumVars];
			double[,] fSmat = new double[nNumVars + 1, nNumVars];
			
			m_MyFx = MyFx;
            F1 = MyFx(nNumVars, ref fX, ref fParam);
			// Build fSmat matrix
			for (k = 0; k < nNumVars; k++)
			{
				fX1[k] = fX[k];
				for (m = 0; m < nNumVars; m++)
				{
					fSmat[k, m] = k == m ? 1 : 0;
				}
			}
			
			sErrorMsg = "";
			nIter = 1;
			j = 1;
			
			do
			{
				nIter++;
				
				// reset row fSmat(nNumVars+1,:)
				for (k = 0; k < nNumVars; k++)
				{
					fSmat[nNumVars, k] = 0;
				}
				
				for (i = 0; i < nNumVars; i++)
				{
					fAlpha = 0;
                    if (!LinSearch_DirectSearch(nNumVars, ref fX, ref fParam, ref fAlpha, ref fSmat, i, 0.1, 0.0001))
					{
						sErrorMsg = "Linear Search failed";
                        break;
					}
					for (k = 0; k < nNumVars; k++)
					{
						fX[k] += fAlpha * fSmat[i, k];
						fSmat[nNumVars, k] = fSmat[nNumVars, k] + fAlpha * fSmat[i, k];
					}
				}
				
				fAlpha = 0;
                if (!LinSearch_DirectSearch(nNumVars, ref fX, ref fX, ref fAlpha, ref fSmat, nNumVars, 0.1, 0.0001))
				{
					sErrorMsg = "Linear Search failed";
					break;
				}
				
				for (k = 0; k < nNumVars; k++)
				{
					fX[k] += fAlpha * fSmat[nNumVars, k];
					fX2[k] = fX[k];
				}
				object temp_object4 = fParam;
				double[] temp_double4 =  (double[]) temp_object4;
				F2 = MyFx(nNumVars, ref fX2, ref temp_double4);
				
				// test end of iterations criteria
				if (Math.Abs(F2 - F1) < fEpsFx)
				{
					break;
				}
				fSum = 0;
				for (k = 0; k < nNumVars; k++)
				{
					fSum += Math.Pow((fX2[k] - fX1[k]), 2);
				}
				fSum = Math.Sqrt(fSum);
				if (fSum < fEpsStep)
				{
					break;
				}
				if (j >= nMaxCycles)
				{
					break;
				}
				
				j++;
				
				// rotate data
				for (k = 0; k < nNumVars; k++)
				{
					fX1[k] = fX2[k];
				}
				F1 = F2;
				
				// copy fSmat matrix
				for (k = 0; k < nNumVars; k++)
				{
					for (m = 0; m < nNumVars; m++)
					{
						fSmat[k, m] = fSmat[k + 1, m];
					}
				}
				
			} while (true);
			
			return F1;
			
		}
		
	}
	
}
